@@ -130,10 +130,77 @@ showdiskusagechanges(Transaction *trans)
   duc[3].path = "/etc";
   transaction_calc_duchanges(trans, duc, 4);
   for (i = 0; i < 4; i++)
-    printf("duchanges %s: %d K  %d inodes\n", duc[i].path, duc[i].kbytes, duc[i].files);
+    if (duc[i].kbytes || duc[i].files)
+      printf("duchanges %s: %lld K  %lld inodes\n", duc[i].path, duc[i].kbytes, duc[i].files);
 }
 #endif
 
+static void
+doshowproof(Solver *solv, Id id, int flags, Queue *lq)
+{
+  Pool *pool = solv->pool;
+  Queue q, qp;
+  int i, j;
+
+  queue_init(&q);
+  queue_init(&qp);
+  solver_get_decisionlist(solv, id, flags | SOLVER_DECISIONLIST_SORTED | SOLVER_DECISIONLIST_WITHINFO | SOLVER_DECISIONLIST_MERGEDINFO, &q);
+  for (i = 0; i < q.count; i += 8)
+    {
+      Id v = q.elements[i];
+      int reason = q.elements[i + 1], bits = q.elements[i + 3], type = q.elements[i + 4];
+      Id from = q.elements[i + 5], to = q.elements[i + 6], dep = q.elements[i + 7];
+      if (reason != SOLVER_REASON_UNSOLVABLE && type == SOLVER_RULE_PKG_SAME_NAME)
+        continue;       /* do not show "obvious" decisions */
+      solver_decisionlist_solvables(solv, &q, i, &qp);
+      if (qp.count)
+        i += qp.count * 8 - 8;
+      if (reason == SOLVER_REASON_UNSOLVABLE)
+        printf("unsolvable: ");
+      else
+        printf("%s %s: ", v < 0 ? "conflicted" : "installed", pool_solvidset2str(pool, &qp));
+      if (type == 0)
+        {
+          printf("%s\n", solver_reason2str(solv, reason));
+          continue;
+        }
+      if (type == SOLVER_RULE_LEARNT && lq)
+        {
+          for (j = 0; j < lq->count; j++)
+            if (lq->elements[j] == q.elements[i + 2])
+              break;
+          if (j < lq->count)
+            {
+              printf("learnt rule #%d\n", j + 1);
+              continue;
+            }
+        }
+      printf("%s\n", solver_decisioninfo2str(solv, bits, type, from, to, dep));
+    }
+  queue_free(&qp);
+  queue_free(&q);
+  printf("\n");
+}
+
+static void
+showproof(Solver *solv, int problem)
+{
+  Queue lq;
+  int i;
+
+  printf("\n");
+  queue_init(&lq);
+  solver_get_learnt(solv, problem, SOLVER_DECISIONLIST_PROBLEM, &lq);
+  for (i = 0; i < lq.count; i++)
+    {
+      printf("Learnt rule #%d:\n", i + 1);
+      doshowproof(solv, lq.elements[i], SOLVER_DECISIONLIST_LEARNTRULE, &lq);
+    }
+  printf("Proof:\n");
+  doshowproof(solv, problem, SOLVER_DECISIONLIST_PROBLEM, &lq);
+  queue_free(&lq);
+}
+
 static Id
 find_repo(const char *name, Pool *pool, struct repoinfo *repoinfos, int nrepoinfos)
 {
@@ -169,7 +236,6 @@ find_repo(const char *name, Pool *pool, struct repoinfo *repoinfos, int nrepoinf
   return 0;
 }
 
-
 #define MODE_LIST        0
 #define MODE_INSTALL     1
 #define MODE_ERASE       2
@@ -227,6 +293,7 @@ main(int argc, char **argv)
   char *rootdir = 0;
   char *keyname = 0;
   int keyname_depstr = 0;
+  int keyname_alldeps = 0;		/* dnf repoquesy --alldeps */
   int debuglevel = 0;
   int answer, acnt = 0;
   char *testcase = 0;
@@ -316,9 +383,15 @@ main(int argc, char **argv)
 	  argc--;
 	  argv++;
 	}
+      else if (argc > 1 && !strcmp(argv[1], "--alldeps"))
+	{
+	  keyname_alldeps = 1;		/* dnf repoquesy --alldeps */
+	  argc--;
+	  argv++;
+	}
       else if (argc > 1 && !strcmp(argv[1], "--depstr"))
 	{
-	  keyname_depstr = 1;
+	  keyname_depstr = 1;	/* do literal matching instead of dep intersection */
 	  argc--;
 	  argv++;
 	}
@@ -539,7 +612,7 @@ main(int argc, char **argv)
 	flags |= SELECTION_FILELIST | (mode == MODE_ERASE ? SELECTION_INSTALLED_ONLY : 0);
       if (keyname && keyname_depstr)
 	flags |= SELECTION_MATCH_DEPSTR;
-      if (!keyname)
+      if (!keyname || keyname_alldeps)
         rflags = selection_make(pool, &job2, argv[i], flags);
       else
         rflags = selection_make_matchdeps(pool, &job2, argv[i], flags, pool_str2id(pool, keyname, 1), 0);
@@ -552,7 +625,7 @@ main(int argc, char **argv)
       if (!job2.count)
 	{
 	  flags |= SELECTION_NOCASE;
-	  if (!keyname)
+	  if (!keyname || keyname_alldeps)
             rflags = selection_make(pool, &job2, argv[i], flags);
 	  else
 	    rflags = selection_make_matchdeps(pool, &job2, argv[i], flags, pool_str2id(pool, keyname, 1), 0);
@@ -574,6 +647,14 @@ main(int argc, char **argv)
         printf("[using file list match for '%s']\n", argv[i]);
       if (rflags & SELECTION_PROVIDES)
 	printf("[using capability match for '%s']\n", argv[i]);
+      if (keyname && keyname_alldeps)
+	{
+	  Queue q;
+	  queue_init(&q);
+	  selection_solvables(pool, &job2, &q);
+	  selection_make_matchsolvablelist(pool, &job2, &q, 0, pool_str2id(pool, keyname, 1), 0);
+	  queue_free(&q);
+	}
       queue_insertn(&job, job.count, job2.count, job2.elements);
       queue_free(&job2);
     }
@@ -741,6 +822,8 @@ rerunsolver:
 		  take = 0;
 		  break;
 		}
+	      if (*ip == 'p')
+		showproof(solv, problem);
 	      if (*ip == 'q')
 		{
 		  printf("Abort.\n");
@@ -773,7 +856,7 @@ rerunsolver:
 #if defined(SUSE)
   showdiskusagechanges(trans);
 #endif
-  printf("install size change: %d K\n", transaction_calc_installsizechange(trans));
+  printf("install size change: %lld K\n", transaction_calc_installsizechange(trans));
   printf("\n");
 
   acnt = solver_alternatives_count(solv);
